<?php
/**
 * CloudImage MediaWiki extension  version 1.0.0
 *	for details please see: https://www.mediawiki.org/wiki/Extension:CloudImage
 *
 * Copyright (c) 2023 Kimagurenote https://kimagurenote.net/
 * License: Revised BSD license http://opensource.org/licenses/BSD-3-Clause
 *
 * Function:
 *	Mediawiki extension that embed images from cloud storage services.
 *
 * Usage:
 *	<img src=https://onedrive.live.com/embed?…" border="0" width="600" height="400" alt="test image" />
 *	for details please see: https://kimagurenote.net/kn/CloudImage
 *
 * Dependency:
 *	MediaWiki 1.35+
 *	If you use MediaWiki 1.34 or below, use $wgAllowImageTag .
 *	https://www.mediawiki.org/wiki/Manual:$wgAllowImageTag
 *
 * Notice!
 *	This extension may be a privacy risk. see:
 *	https://www.mediawiki.org/wiki/Manual:$wgAllowExternalImages#Why_disallow_external_images?
 *
 * History:
 * 2023.07.06 Version 1.0.0
 *	1st test (support only OneDrive and Flickr)
 *
 * @file
 * @ingroup Extensions
 * @author Kimagurenote
 * @copyright © 2023 Kimagurenote
 * @license The BSD 3-Clause License
 */

if( !defined( 'MEDIAWIKI' ) ) {
	echo( "This file is an extension to the MediaWiki software and cannot be used standalone.\n" );
	die( 1 );
}

/* global variables */
global $wgCloudImageTag, $wgCloudImageAllowedDomains, $wgCloudImageDefaultAttrs, $wgCloudImageFigAttrs, $wgCloudImageFigBoarder, $wgCloudImageCaptionAttrs;

/* tag name */
if( empty( $wgCloudImageTag ) ) {
	$wgCloudImageTag = "cimg";
} else {
	$wgCloudImageTag = strtolower( $wgCloudImageTag );
}

/* arrowed URLs and special functions (option) */
$wgCloudImageAllowedDomains['onedrive.live.com'] = '';
$wgCloudImageAllowedDomains['live.staticflickr.com'] = '';

/* default attributes and values to set if not given */
$wgCloudImageDefaultAttrs['border'] = "0";

/* attributes for <figure> and <figcaption> */
$wgCloudImageFigAttrs['frameclass'] = "class";
$wgCloudImageFigAttrs['frameid'] = "id";
$wgCloudImageFigAttrs['framestyle'] = "style";
$wgCloudImageFigBoarder = 1;
$wgCloudImageCaptionAttrs['captionclass'] = "class";
$wgCloudImageCaptionAttrs['captionid'] = "id";
$wgCloudImageCaptionAttrs['captionstyle'] = "style";

class CloudImage {

	/**
	 * Register all the new tags with the Parser.
	 *
	 * @param Parser &$parser
	 */
	public static function onParserFirstCallInit( Parser $parser ) {
		global $wgCloudImageTag;

		if( !empty( $wgCloudImageTag ) ) {
			$parser->setHook( $wgCloudImageTag, [ self::class, 'embedCloudImage' ] );
		}
	}

	/**
	 * @param string $src
	 * @param array $argv
	 * return string
	 *	""	failed (not processed)
	 */
	private static function generateImgTag( $src, $argv, $caption, $parser ) {
		global $wgCloudImageDefaultAttrs, $wgCloudImageAllowedAttrs, $wgCloudImageFigAttrs, $wgCloudImageFigBoarder, $wgCloudImageCaptionAttrs;

		if( empty( $src ) ) {
			return '';
		};
		$params['src'] = $src;

		array_change_key_case( $argv, CASE_LOWER );

		/* get <a> attributes */
		if( !empty( $argv['href'] ) ) {
			$href = $argv['href'];
			unset( $argv['href'] );
		}

		/* get <figure> attributes */
		if( !empty( $wgCloudImageFigAttrs ) ) {
			foreach( $wgCloudImageFigAttrs as $k => $v ) {
				if( !empty( $argv[$k] ) ) {
					$figparams[$v] = $argv[$k];
					unset( $argv[$k] );
				}
			}
		}

		/* get <figcaption> attributes */
		if( !empty( $wgCloudImageCaptionAttrs ) ) {
			foreach( $wgCloudImageCaptionAttrs as $k => $v ) {
				if( !empty( $argv[$k] ) ) {
					$captionparams[$v] = $argv[$k];
					unset( $argv[$k] );
				}
			}
		}

		/* set default attrs at first */
		if( !empty( $wgCloudImageDefaultAttrs ) ) {
			foreach( $wgCloudImageDefaultAttrs as $k => $v ) {
				if( empty( $argv[$k] ) ) {
					$params[$k] = $v;
				} else {
					$params[$k] = $argv[$k];
					unset( $argv[$k] );
				}
			}
		}

		if( empty( $wgCloudImageAllowedAttrs ) ) {
			foreach( $argv as $k => $v ) {
					$params[$k] = $v;
			}
		} else {
			foreach( $wgCloudImageAllowedAttrs as $k ) {
				if( isset( $argv[$k] ) ) {
					$params[$k] = $attr[$k];
				}
			}
		}
		$ret = Html::rawElement( 'img', $params );

		if( !empty( $href ) ) {
			$ret = Html::rawElement( 'a', [ 'href' => $href ], $ret );
		}

		if( !empty( $caption ) ) {
			if( empty( $figparams['class'] ) ) {
				$figparams['class'] = "cimg";
			} else {
				$figparams['class'] = "cimg " . $figparams['class'];
			}
			if( empty( $captionparams['class'] ) ) {
				$captionparams['class'] = "cimgcaption";
			} else {
				$captionparams['class'] = "cimgcaption " . $captionparams['class'];
			}
			if( !empty( $params['width'] ) ) {
				if( empty( $figparams['style'] ) ) {
					$figparams['style'] = "";
				} else {
					$figparams['style'] .= " ";
				}
				$figparams['style'] .= "width: " . $params['width'] + $wgCloudImageFigBoarder * 2 . "px;";
			}
			$ret .= Html::rawElement( 'figcaption', $captionparams, $parser->internalParse( $caption, false ) );
			$ret = Html::rawElement( 'figure', $figparams, $ret );
		}

		return $ret;
	}


	/**
	 * @link https://www.mediawiki.org/wiki/Manual:Hooks/LinkerMakeExternalLink
	 * @param string $input
	 * @param array $argv
	 * @param Parser $parser
	 *
	 * @return string
	 */
	public static function embedCloudImage( $input, $argv, $parser ) {

		/* not process external link with different caption */
		$src = $argv['src'];
		if( empty( $src ) ) {
			return "<!-- CloudImage: No src found! -->";
		}

		/* call process if no restriction */
		if( empty( $wgCloudImageAllowedDomains ) ) {
			return self::generateImgTag( $src, $argv, $input, $parser );
		} else {
			/* find url and call process */
			foreach( $wgCloudImageAllowedDomains as $addr => $func ) {
				if( preg_match( "/^https?:\/\/" . $addr . "/", $src ) ) {
					if( empty( $func ) ) {
						return self::generateImgTag( $src, $argv, $input, $parser );
					} else {
						return self::$func( $src, $argv, $input, $parser );
					}
				}
			}
		}

		/* not allowed domain */
		return "";
	}
}
?>
